#include <ESP8266WiFi.h> //Library for wifi 
#include <MQTT.h>//library for mqtt
#include "StringSplitter.h"//library for splitting strings
#include <Adafruit_NeoPixel.h>// library for neopixels

#define PIN D4        //Output pin 
#define NUMPIXELS 12  //Number of neopixels

//initial neopixels
Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

const char ssid[] = "StudentResidences";// wifi network
const char pass[] = ""; //wifi password (empty for open networks)
WiFiClient net; //wifi client 
MQTTClient client;//mqtt client
unsigned long lastMillis = 0; //variable to store the last time a message was sent
int ledPin = D4; //define led pin

//establish wifi and mqtt connection
void connect() {
  Serial.print("checking wifi...");
  while (WiFi.status() != WL_CONNECTED) { //keeping checking till connected to wifi
    Serial.print(".");
    delay(1000);
  }
  Serial.print("\nconnecting...");
  while (!client.connect("CatrionaPaterson", "", "")) { //conect to mqtt broker
    Serial.print(".");
    delay(1000);
  }
  Serial.println("\nconnected!");
  client.subscribe("catpatersonemo/IR"); //subscribe to mqtt topic
 
}

//function to handle incoming mqtt messages
void messageReceived(String &topic, String &payload) {
  //Connects to the topic on the MQTT broker, and returns the most recent data in that topic.
  //Data is held in the variable "payload"

  //split the recieved payloadusing a comma as a delimiter
  Serial.println("incoming: " + topic + " - " + payload);
  StringSplitter *splitter = new StringSplitter(payload, ',', 2); //Splitting the data into two pieces making them two seperate variables
  int itemCount = splitter->getItemCount();// count the number of itams
  Serial.println("Item count: " + String(itemCount));
  
  // variables to store the split values
  String val0, val1;
  
  for (int i = 0; i < itemCount; i++) {
    String item = splitter->getItemAtIndex(i); //get item from the split string
    if (i == 0) {
      val0 = item; //first value
    }
    if (i == 1) {
      val1 = item; //second value
    }
  }
//turning them into integers
  int ir = val0.toInt();  
  int visible = val1.toInt();
  Serial.println(ir);
  Serial.println(visible);

//giving the integers map perameters
  int ledRed = map(ir, 0, 1300, 0, 255); 
  int ledBlue = map(visible, 0, 270, 0, 255);

//reading the potentiometer
  int potValue = analogRead(A0);

// when the potentiometer is below a reading of 341 use the MQTT data to give the neopixels values
  if (potValue < 341) {
    //use MQTT data
    pixels.clear();                        // Set all pixel colors to 'off'
    for (int i = 0; i < NUMPIXELS; i++) {  // For each pixel...

      pixels.setPixelColor(i, pixels.Color(ledRed, 0, ledBlue));

      pixels.show();  // Send the updated pixel colors to the hardware.
    }

//if the potentiometer reads between 341 and 682 it sets the neopixels to set constant value
  } else if (potValue > 341 && potValue < 682) {
    //show static colour
    // pixels.clear();                        // Set all pixel colors to 'off'
    for (int i = 0; i < NUMPIXELS; i++) {  // For each pixel...

      pixels.setPixelColor(i, pixels.Color(0, 0, 100));

      pixels.show();  // Send the updated pixel colors to the hardware.
    }

// if the potentiometer is above 682 then it changes to a another set constant value
  } else if (potValue >682) {
    for (int i = 0; i < NUMPIXELS; i++) {  // For each pixel...

      pixels.setPixelColor(i, pixels.Color(100, 0, 0));

      pixels.show();  // Send the updated pixel colors to the hardware.
    }
  }
}

// setup function runs once when the device starts
void setup() {
  Serial.begin(115200); //initialize serial communication
  pixels.begin();  // initialize NeoPixel strip object (REQUIRED)
  WiFi.begin(ssid, pass); //connect to wifi
  client.begin("test.mosquitto.org", net);//connect to mqtt
  client.onMessage(messageReceived);//set callback for incoming messages
  connect(); //connect to wifi and mqtt
  pinMode(D4, OUTPUT);//set led pin as output
}

//loop function runs continuously
void loop() {
  client.loop(); //keep mqtt client connected
  delay(10);  // short delay to maintain wifi stability

  if (!client.connected()) { //if client gets disconnected, reconnect
    connect();
  }
}
